﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace RIAcontrols
{
    [DefaultProperty("Source")]
    [ToolboxData("<{0}:Silverlight runat=server></{0}:Silverlight>")]
    public class Silverlight : WebControl, INamingContainer
    {
        #region Public Properties

        [Bindable(true)]
        [Category("Misc")]
        [DefaultValue("")]
        [Localizable(true)]
        public String Source
        {
            get
            {
                String s = (String)ViewState["Source"];
                return ((s == null) ? String.Empty : s);
            }

            set
            {
                ViewState["Source"] = value;
            }
        }

        [Bindable(true)]
        [Category("Misc")]
        [DefaultValue("3.0.40620.0")]
        [Localizable(true)]
        public String MinRuntimeVersion
        {
            get
            {
                String s = (String)ViewState["MinRuntimeVersion"];
                return ((s == null) ? "3.0.40620.0" : s);
            }

            set
            {
                ViewState["MinRuntimeVersion"] = value;
            }
        }

        [Bindable(true)]
        [Category("Misc")]
        [DefaultValue(null)]
        [Localizable(true)]
        public String InitParams
        {
            get
            {
                String s = (String)ViewState["InitParams"];
                return ((s == null) ? String.Empty : s);
            }

            set
            {
                ViewState["InitParams"] = value;
            }
        }

        [Bindable(true)]
        [Category("Misc")]
        [DefaultValue(true)]
        [Localizable(true)]
        public Boolean EnableHtmlAccess
        {
            get
            {
                object o = ViewState["EnableHtmlAccess"];
                if (o == null)
                    return true;

                return (Boolean)o;
            }

            set
            {
                ViewState["EnableHtmlAccess"] = value;
            }
        }

        [Bindable(true)]
        [Category("Misc")]
        [DefaultValue("")]
        [Localizable(true)]
        public String OnSilverlightError
        {
            get
            {
                String s = (String)ViewState["OnSilverlightError"];
                return ((s == null) ? String.Empty : s);
            }

            set
            {
                ViewState["OnSilverlightError"] = value;
            }
        }

        [Bindable(true)]
        [Category("Misc")]
        [DefaultValue(true)]
        [Localizable(true)]
        public Boolean AutoUpgrade
        {
            get
            {
                object o = ViewState["AutoUpgrade"];
                if (o == null)
                    return true;

                return (Boolean)o;
            }

            set
            {
                ViewState["AutoUpgrade"] = value;
            }
        }

        [Bindable(true)]
        [Category("Misc")]
        [DefaultValue(false)]
        [Localizable(true)]
        public Boolean Windowless
        {
            get
            {
                object o = ViewState["Windowless"];
                if (o == null)
                    return false;

                return (Boolean)o;
            }

            set
            {
                ViewState["Windowless"] = value;
            }
        }

        [PersistenceMode(PersistenceMode.InnerProperty)]
        public ITemplate NoSilverlightTemplate
        {
            get;
            set;
        }

        public override ControlCollection Controls
        {
            get
            {
                this.EnsureChildControls();
                return base.Controls;
            }
        }

        #endregion

        #region Protected Properties
        protected override HtmlTextWriterTag TagKey
        {
            get
            {
                return HtmlTextWriterTag.Object;
            }
        }

        #endregion

        #region Protected Methods
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            Int32 MajorVersion;
            String MimeType;
            Int32 MajorLenght = this.MinRuntimeVersion.IndexOf(".");

            // Detect Silverlight Major Version required
            if (MajorLenght > 0)
            {
                try
                {
                    MajorVersion = Convert.ToInt32(this.MinRuntimeVersion.Substring(0, MajorLenght));
                }
                catch
                {
                    MajorVersion = 3;
                }
            }
            else
            {
                MajorVersion = 3;
            }

            if (MajorVersion == 1)
            {
                MimeType = "application/x-silverlight";
            }
            else
            {
                MimeType = "application/x-silverlight-2";
            }

            // Write attributes
            writer.AddAttribute(HtmlTextWriterAttribute.Id, this.ClientID);
            writer.AddAttribute(HtmlTextWriterAttribute.Name, this.ID);
            writer.AddAttribute(HtmlTextWriterAttribute.Width, this.Width.ToString());
            writer.AddAttribute(HtmlTextWriterAttribute.Height, this.Height.ToString());
            writer.AddAttribute(HtmlTextWriterAttribute.Type, MimeType);
            writer.AddAttribute("data", String.Format("data:{0},", MimeType));

        }

        protected override void RenderContents(HtmlTextWriter output)
        {
            output.Indent++;

            // Source
            AddParam(output, "source", this.ResolveUrl(this.Source));

            // EnableHtmlAccess
            AddParam(output, "enablehtmlaccess", this.EnableHtmlAccess.ToString());

            // AutoUpgrade
            AddParam(output, "autoUpgrade", this.AutoUpgrade.ToString());

            // MinRuntimeVersion
            AddParam(output, "minRuntimeVersion", this.MinRuntimeVersion);

            // Windowless
            AddParam(output, "windowless", this.Windowless.ToString());

            // OnSilverlightError
            if (this.OnSilverlightError != String.Empty)
            {
                AddParam(output, "onerror", this.OnSilverlightError);
            }

            // BackColor
            if (!this.BackColor.IsEmpty)
            {
                AddParam(output, "background", String.Format("#{0:X2}{1:X2}{2:X2}", this.BackColor.R, this.BackColor.G, this.BackColor.B));
            }

            // InitParams
            if (this.InitParams != String.Empty)
            {
                AddParam(output, "initParams", this.InitParams);
            }

            base.RenderChildren(output);

        }

        protected override void CreateChildControls()
        {
            if (NoSilverlightTemplate != null)
            {
                Control NoSilverlightTemplateHost = new Control();
                NoSilverlightTemplate.InstantiateIn(NoSilverlightTemplateHost);
                Controls.Add(NoSilverlightTemplateHost);
            }

            base.CreateChildControls();
        }

        #endregion

        #region Public Methods
        public void ClearInitParams()
        {
            this.InitParams = String.Empty;
        }

        public String GetInitParam(String ParamKey)
        {
            Dictionary<String, String> dParams = this.GetInitParams();

            if (dParams.ContainsKey(ParamKey))
            {
                return dParams[ParamKey];
            }
            else
            {
                return String.Empty;
            }
        }

        public void SetInitParam(String ParamKey, String ParamValue)
        {
            Dictionary<String, String> dParams = this.GetInitParams();

            if (dParams.ContainsKey(ParamKey))
            {
                dParams[ParamKey] = ParamValue;
            }
            else
            {
                dParams.Add(ParamKey, ParamValue);
            }

            SetInitParams(dParams);
        }

        public void RemoveInitParams(String ParamKey)
        {
            Dictionary<String, String> dParams = this.GetInitParams();

            if (dParams.ContainsKey(ParamKey))
            {
                dParams.Remove(ParamKey);
                SetInitParams(dParams);
            }
        }

        public Dictionary<String, String> GetInitParams()
        {
            Dictionary<String, String> Result = new Dictionary<String, String>();

            foreach (String Param in this.InitParams.Split(new Char[] { ',' }))
            {
                if (!String.IsNullOrEmpty(Param) && Param.Contains('='))
                {
                    String Key = Param.Substring(0, Param.IndexOf('=')).Trim();
                    String Value = Param.Substring(Param.IndexOf('=') + 1).Trim();
                    Result.Add(Key, Value);
                }
            }

            return Result;
        }

        public void SetInitParams(Dictionary<String, String> dVars)
        {
            String sFlashVars = String.Empty;

            foreach (String Key in dVars.Keys)
            {
                sFlashVars += String.Format("{0}={1},", Key, dVars[Key]);
            }

            if (sFlashVars.Length > 0)
            {
                sFlashVars = sFlashVars.Substring(0, sFlashVars.Length - 1);
            }

            InitParams = sFlashVars;
        }

        #endregion

        #region Private Methods
        private void AddParam(HtmlTextWriter writer, String ParamName, String ParamValue)
        {
            writer.WriteBeginTag("param");
            writer.WriteAttribute("name", ParamName);
            writer.WriteAttribute("value", ParamValue);
            writer.WriteLine(HtmlTextWriter.SelfClosingTagEnd);
        }

        #endregion

        #region Public Enums
        public enum WModes
        {
            window,
            opaque,
            transparent
        }

        public enum Alignments
        {
            middle,
            left,
            right,
            top
        }

        #endregion
    }
}
